@@ -18,12 +18,13 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
-;; This package conversts Mocklisp code written under a Gosling or UniPress
+;; This package converts Mocklisp code written under a Gosling or UniPress
 ;; Emacs for use with GNU Emacs.  The translated code will require runtime
 ;; support from the mlsupport.el equivalent.
 
@@ -110,19 +111,19 @@
 		       (if (looking-at "setq[ \t\n]+buffer-modified-p")
 			   (replace-match "set-buffer-modified-p"))))
 
-(ml-expansion 'while '(lambda ()
-			 (let ((end (progn (forward-sexp 2) (point-marker)))
-			       (start (progn (forward-sexp -1) (point))))
-			   (let ((cond (buffer-substring start end)))
-			     (cond ((equal cond "1")
-				    (delete-region (point) end)
-				    (insert "t"))
-				   (t
-				    (insert "(not (zerop ")
-				    (goto-char end)
-				    (insert "))")))
-			     (set-marker end nil)
-			     (goto-char start)))))
+;;(ml-expansion 'while '(lambda ()
+;;			 (let ((end (progn (forward-sexp 2) (point-marker)))
+;;			       (start (progn (forward-sexp -1) (point))))
+;;			   (let ((cond (buffer-substring start end)))
+;;			     (cond ((equal cond "1")
+;;				    (delete-region (point) end)
+;;				    (insert "t"))
+;;				   (t
+;;				    (insert "(not (zerop ")
+;;				    (goto-char end)
+;;				    (insert "))")))
+;;			     (set-marker end nil)
+;;			     (goto-char start)))))
 
 (ml-expansion 'arg "ml-arg")
 (ml-expansion 'nargs "ml-nargs")
@@ -194,6 +195,7 @@
 (ml-expansion 'get-tty-no-blanks-input "read-no-blanks-input")
 (ml-expansion 'get-tty-key "read-key")
 
+(ml-expansion 'concat "ml-concat")
 (ml-expansion 'c= "char-equal")
 (ml-expansion 'goto-character "goto-char")
 (ml-expansion 'substr "ml-substr")
