@@ -15,7 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+Boston, MA 02111-1307, USA.  */
 
 
 /* Miscellanea.  */
@@ -37,7 +38,7 @@ extern int message_buf_print;
    having miscellaneous random variables scattered about.  */
 
 enum output_method
-{ output_termcap, output_x_window, output_msdos_raw };
+{ output_termcap, output_x_window, output_msdos_raw, output_win32 };
 
 struct frame
 {
@@ -48,14 +49,18 @@ struct frame
      Only EMACS_INT values can be intermixed with them.
      That ensures they are all aligned normally.  */
 
-  /* Name of this frame: a Lisp string.  See also `explicit_name'
-     and `namebuf'.  */
+  /* Name of this frame: a Lisp string.  It is used for looking up resources,
+     as well as for the title in some cases.  */
   Lisp_Object name;
 
   /* The name to use for the icon, the last time
      it was refreshed.  nil means not explicitly specified.  */
   Lisp_Object icon_name;
 
+  /* This is the frame title specified explicitly, if any.
+     Usually it is nil.  */
+  Lisp_Object title;
+
   /* The frame which should receive keystrokes that occur in this
      frame, or nil if they should go to the frame itself.  This is
      usually nil, but if the frame is minibufferless, we can use this
@@ -101,9 +106,11 @@ struct frame
   Lisp_Object scroll_bars;
   Lisp_Object condemned_scroll_bars;
 
-  /* List of elements to display in the menu bar.
-     The elements have the form (KEY STRING . nil) to start;
-     when they are displayed, the hpos of the left edge goes in the cddr.  */
+  /* Vector describing the items to display in the menu bar.
+     Each item has four elements in this vector.
+     They are KEY, STRING, SUBMAP, and HPOS.
+     (HPOS is not used in when the X toolkit is in use.)
+     There are four additional elements of nil at the end, to terminate.  */
   Lisp_Object menu_bar_items;
 
   /* Alist of elements (FACE-NAME . FACE-VECTOR-DATA).  */
@@ -171,9 +178,11 @@ struct frame
   enum output_method output_method;
 
   /* A structure of auxiliary data used for displaying the contents.
-     struct x_display is used for X window frames;
-     it is defined in xterm.h.  */
-  union display { struct x_display *x; int nothing; } display;
+     struct x_output is used for X window frames;
+     it is defined in xterm.h.  
+     struct win32_output is used for Win32 window frames;
+     it is defined in w32term.h.  */
+  union output_data { struct x_output *x; struct win32_output *win32; int nothing; } output_data;
 
 #ifdef MULTI_KBOARD
   /* A pointer to the kboard structure associated with this frame.
@@ -185,7 +194,7 @@ struct frame
   /* Number of lines of menu bar.  */
   int menu_bar_lines;
 
-#ifdef USE_X_TOOLKIT
+#if defined (USE_X_TOOLKIT) || defined (HAVE_NTGUI)
   /* Nonzero means using a menu bar that comes from the X toolkit.  */
   int external_menu_bar;
 #endif
@@ -279,6 +288,10 @@ struct frame
 
   /* The baud rate that was used to calculate costs for this frame.  */
   int cost_calculation_baud_rate;
+
+  /* Nonzero if the mouse has moved on this display
+     since the last time we checked.  */
+  char mouse_moved;
 };
 
 #ifdef MULTI_KBOARD  /* Note that MULTI_KBOARD implies MULTI_FRAME */
@@ -296,9 +309,24 @@ typedef struct frame *FRAME_PTR;
 
 #define WINDOW_FRAME(w) (w)->frame
 
-#define FRAME_LIVE_P(f) ((f)->display.nothing != 0)
-#define FRAME_TERMCAP_P(f) ((f)->output_method == output_termcap)
 #define FRAME_X_P(f) ((f)->output_method == output_x_window)
+#define FRAME_WIN32_P(f) ((f)->output_method == output_win32)
+#define FRAME_MSDOS_P(f) ((f)->output_method == output_msdos_raw)
+
+/* FRAME_WINDOW_P tests whether the frame is a window, and is
+   defined to be the predicate for the window system being used.  */
+#ifdef HAVE_X_WINDOWS
+#define FRAME_WINDOW_P(f) FRAME_X_P (f)
+#endif
+#ifdef HAVE_NTGUI
+#define FRAME_WINDOW_P(f) FRAME_WIN32_P (f)
+#endif
+#ifndef FRAME_WINDOW_P
+#define FRAME_WINDOW_P(f) (0)
+#endif
+
+#define FRAME_LIVE_P(f) ((f)->output_data.nothing != 0)
+#define FRAME_TERMCAP_P(f) ((f)->output_method == output_termcap)
 #define FRAME_MINIBUF_ONLY_P(f) \
   EQ (FRAME_ROOT_WINDOW (f), FRAME_MINIBUF_WINDOW (f))
 #define FRAME_HAS_MINIBUF_P(f) ((f)->has_minibuffer)
@@ -310,7 +338,7 @@ typedef struct frame *FRAME_PTR;
 #define FRAME_NEW_HEIGHT(f) (f)->new_height
 #define FRAME_NEW_WIDTH(f) (f)->new_width
 #define FRAME_MENU_BAR_LINES(f) (f)->menu_bar_lines
-#ifdef USE_X_TOOLKIT
+#if defined (USE_X_TOOLKIT) || defined (HAVE_NTGUI)
 #define FRAME_EXTERNAL_MENU_BAR(f) (f)->external_menu_bar
 #else
 #define FRAME_EXTERNAL_MENU_BAR(f) 0
@@ -442,6 +470,7 @@ extern FRAME_PTR selected_frame;
 extern FRAME_PTR last_nonminibuf_frame;
 
 #define FRAME_LIVE_P(f) 1
+#define FRAME_MSDOS_P(f) 0
 #ifdef MSDOS
 /* The following definitions could also be used in the non-MSDOS case,
    but the constants below lead to better code.  */
@@ -451,6 +480,7 @@ extern FRAME_PTR last_nonminibuf_frame;
 #define FRAME_TERMCAP_P(f) 1
 #define FRAME_X_P(f) 0
 #endif
+#define FRAME_WINDOW_P(f) FRAME_X_P (f)
 #define FRAME_MINIBUF_ONLY_P(f) 0
 #define FRAME_HAS_MINIBUF_P(f) 1
 #define FRAME_CURRENT_GLYPHS(f) (the_only_frame.current_glyphs)
