@@ -10,7 +10,7 @@ NOTE-START
 Use -opsystem=osf1 for OSF/1, and -opsystem=bsd4-3 otherwise.
 NOTE-END  */
 
-#undef BIG_ENDIAN
+#undef WORDS_BIG_ENDIAN
 #undef LIB_STANDARD
 #undef START_FILES
 #undef COFF
@@ -18,9 +18,14 @@ NOTE-END  */
 #define MAIL_USE_FLOCK
 #define HAVE_UNION_WAIT
 
+
+#ifdef MACH
+#define START_FILES pre-crt0.o /usr/lib/crt0.o
+#else
 /* This line starts being needed with ultrix 4.0.  */
 /* You must delete it for version 3.1.  */
 #define START_FILES pre-crt0.o /usr/lib/cmplrs/cc/crt0.o
+#endif
 
 /* Supposedly the following will overcome a kernel bug.  */
 #undef LD_SWITCH_MACHINE
@@ -37,7 +42,7 @@ NOTE-END  */
 #define SYSTEM_MALLOC
 #endif
 
-/* Override what m-mips.h says about this.  */
+/* Override what mips.h says about this.  */
 #undef LINKER
 
 /* Ultrix 4.2 (perhaps also 4.1) implements O_NONBLOCK
@@ -45,16 +50,11 @@ NOTE-END  */
    and it causes hanging in read_process_output.  */
 #define BROKEN_O_NONBLOCK
 
-#ifdef OSF1
+#if defined (OSF1) || defined (MACH)
 #undef C_ALLOCA
 #define HAVE_ALLOCA
 #endif
 
-#ifndef OSF1
-/* Ultrix saves the time zone in core; must clear it.  */
-#define LOCALTIME_CACHE
-#endif
-
 /* mcc@timessqr.gc.cuny.edu says this makes Emacs work with DECnet.  */
 #ifdef HAVE_LIBDNET
 #define LIBS_MACHINE -ldnet
@@ -64,6 +64,7 @@ NOTE-END  */
 #undef KERNEL_FILE
 #define KERNEL_FILE "/vmunix"
 
+#ifndef MACH
 /* Jim Wilson writes:
    [...] The X11 include files that Dec distributes with Ultrix
    are bogus.
@@ -85,10 +86,11 @@ NOTE-END  */
    also solve the problem, but I doubt you can convince everyone to do this. */
 /* Addendum: the MIT X11 distribution neglects to define certain symbols
    when NeedFunctionPrototypes is 0, but still tries to use them when
-   NeedVarargsProrotypes is 1 (which is its default value).  So if we're
+   NeedVarargsPrototypes is 1 (which is its default value).  So if we're
    going to disable non-variadic prototypes, we also need to disable
    variadic prototypes.  --kwzh@gnu.ai.mit.edu */
 #define C_SWITCH_X_MACHINE -DNeedFunctionPrototypes=0 -DNeedVarargsPrototypes=0
+#endif
 
 /* Enable a fix in process.c.  */
 #define SET_CHILD_PTY_PGRP
